#
# PySNMP MIB module INTEGRATED-SERVICES-MIB (http://snmplabs.com/pysmi)
# ASN.1 source file://./INTEGRATED-SERVICES-MIB.mib
# Produced by pysmi-1.1.10 at Mon Oct  9 12:14:08 2023
# On host simtve-ThinkPad-X1-Carbon-Gen-9 platform Linux version 5.15.0-84-generic by user simon
# Using Python version 3.9.18 (main, Aug 25 2023, 13:20:04) 
#
Integer, ObjectIdentifier, OctetString = mibBuilder.importSymbols("ASN1", "Integer", "ObjectIdentifier", "OctetString")
NamedValues, = mibBuilder.importSymbols("ASN1-ENUMERATION", "NamedValues")
ConstraintsUnion, ValueRangeConstraint, ValueSizeConstraint, ConstraintsIntersection, SingleValueConstraint = mibBuilder.importSymbols("ASN1-REFINEMENT", "ConstraintsUnion", "ValueRangeConstraint", "ValueSizeConstraint", "ConstraintsIntersection", "SingleValueConstraint")
ifIndex, InterfaceIndex = mibBuilder.importSymbols("IF-MIB", "ifIndex", "InterfaceIndex")
NotificationGroup, ObjectGroup, ModuleCompliance = mibBuilder.importSymbols("SNMPv2-CONF", "NotificationGroup", "ObjectGroup", "ModuleCompliance")
IpAddress, ModuleIdentity, Bits, MibIdentifier, Counter64, Integer32, iso, TimeTicks, NotificationType, Unsigned32, mib_2, MibScalar, MibTable, MibTableRow, MibTableColumn, ObjectIdentity, Gauge32, Counter32 = mibBuilder.importSymbols("SNMPv2-SMI", "IpAddress", "ModuleIdentity", "Bits", "MibIdentifier", "Counter64", "Integer32", "iso", "TimeTicks", "NotificationType", "Unsigned32", "mib-2", "MibScalar", "MibTable", "MibTableRow", "MibTableColumn", "ObjectIdentity", "Gauge32", "Counter32")
RowStatus, TestAndIncr, DisplayString, TextualConvention, TruthValue, TimeInterval = mibBuilder.importSymbols("SNMPv2-TC", "RowStatus", "TestAndIncr", "DisplayString", "TextualConvention", "TruthValue", "TimeInterval")
intSrv = ModuleIdentity((1, 3, 6, 1, 2, 1, 52))
if mibBuilder.loadTexts: intSrv.setLastUpdated('9511030500Z')
if mibBuilder.loadTexts: intSrv.setOrganization('IETF Integrated Services Working Group')
if mibBuilder.loadTexts: intSrv.setContactInfo(' Fred Baker Postal: Cisco Systems 519 Lado Drive Santa Barbara, California 93111 Tel: +1 805 681 0115 E-Mail: fred@cisco.com John Krawczyk Postal: ArrowPoint Communications 235 Littleton Road Westford, Massachusetts 01886 Tel: +1 508 692 5875 E-Mail: jjk@tiac.net')
if mibBuilder.loadTexts: intSrv.setDescription('The MIB module to describe the Integrated Services Protocol')
intSrvObjects = MibIdentifier((1, 3, 6, 1, 2, 1, 52, 1))
intSrvGenObjects = MibIdentifier((1, 3, 6, 1, 2, 1, 52, 2))
intSrvNotifications = MibIdentifier((1, 3, 6, 1, 2, 1, 52, 3))
intSrvConformance = MibIdentifier((1, 3, 6, 1, 2, 1, 52, 4))
class SessionNumber(TextualConvention, Integer32):
    description = 'The Session Number convention is used for numbers identifying sessions or saved PATH or RESV information. It is a number in the range returned by a TestAndIncr variable, having no protocol meaning whatsoever but serving instead as simple identifier. The alternative was a very complex instance or instance object that became unwieldy.'
    status = 'current'
    subtypeSpec = Integer32.subtypeSpec + ValueRangeConstraint(0, 2147483647)

class Protocol(TextualConvention, Integer32):
    description = 'The value of the IP Protocol field of an IP Datagram Header. This identifies the protocol layer above IP. For example, the value 6 is used for TCP and the value 17 is used for UDP. The values of this field are defined in the As- signed Numbers RFC.'
    status = 'current'
    displayHint = 'd'
    subtypeSpec = Integer32.subtypeSpec + ValueRangeConstraint(1, 255)

class SessionType(TextualConvention, Integer32):
    description = "The value of the C-Type field of a Session ob- ject, as defined in the RSVP specification. This value determines the lengths of octet strings and use of certain objects such as the 'port' variables. If the C-Type calls for an IP6 address, one would expect all source, des- tination, and next/previous hop addresses to be 16 bytes long, and for the ports to be UDP/TCP port numbers, for example."
    status = 'current'
    subtypeSpec = Integer32.subtypeSpec + ValueRangeConstraint(1, 255)

class Port(TextualConvention, OctetString):
    description = 'The value of the UDP or TCP Source or Destina- tion Port field, a virtual destination port or generalized port identifier used with the IPSEC Authentication Header or Encapsulating Security Payload, or other session discriminator. If it is not used, the value should be of length 0. This pair, when coupled with the IP Addresses of the source and destination system and the IP protocol field, uniquely identifies a data stream.'
    status = 'current'
    subtypeSpec = OctetString.subtypeSpec + ValueSizeConstraint(2, 4)

class MessageSize(TextualConvention, Integer32):
    description = 'The size of a message in bytes. This is used to specify the minimum and maximum size of a message along an integrated services route.'
    status = 'current'
    displayHint = 'd'
    subtypeSpec = Integer32.subtypeSpec + ValueRangeConstraint(0, 2147483647)

class BitRate(TextualConvention, Integer32):
    description = 'The rate, in bits/second, that data may move in the context. Applicable contexts minimally include the speed of an interface or virtual circuit, the data rate of a (potentially aggre- gated) data flow, or the data rate to be allo- cated for use by a flow.'
    status = 'current'
    displayHint = 'd'
    subtypeSpec = Integer32.subtypeSpec + ValueRangeConstraint(0, 2147483647)

class BurstSize(TextualConvention, Integer32):
    description = 'The number of octets of IP Data, including IP Headers, that a stream may send without concern for policing.'
    status = 'current'
    displayHint = 'd'
    subtypeSpec = Integer32.subtypeSpec + ValueRangeConstraint(0, 2147483647)

class QosService(TextualConvention, Integer32):
    description = 'The class of service in use by a flow.'
    status = 'current'
    subtypeSpec = Integer32.subtypeSpec + ConstraintsUnion(SingleValueConstraint(1, 2, 5))
    namedValues = NamedValues(("bestEffort", 1), ("guaranteedDelay", 2), ("controlledLoad", 5))

intSrvIfAttribTable = MibTable((1, 3, 6, 1, 2, 1, 52, 1, 1), )
if mibBuilder.loadTexts: intSrvIfAttribTable.setStatus('current')
if mibBuilder.loadTexts: intSrvIfAttribTable.setDescription("The reservable attributes of the system's in- terfaces.")
intSrvIfAttribEntry = MibTableRow((1, 3, 6, 1, 2, 1, 52, 1, 1, 1), ).setIndexNames((0, "IF-MIB", "ifIndex"))
if mibBuilder.loadTexts: intSrvIfAttribEntry.setStatus('current')
if mibBuilder.loadTexts: intSrvIfAttribEntry.setDescription('The reservable attributes of a given inter- face.')
intSrvIfAttribAllocatedBits = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 1, 1, 1), BitRate()).setUnits('Bits per second').setMaxAccess("readonly")
if mibBuilder.loadTexts: intSrvIfAttribAllocatedBits.setStatus('current')
if mibBuilder.loadTexts: intSrvIfAttribAllocatedBits.setDescription('The number of bits/second currently allocated to reserved sessions on the interface.')
intSrvIfAttribMaxAllocatedBits = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 1, 1, 2), BitRate()).setUnits('Bits per second').setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvIfAttribMaxAllocatedBits.setStatus('current')
if mibBuilder.loadTexts: intSrvIfAttribMaxAllocatedBits.setDescription('The maximum number of bits/second that may be allocated to reserved sessions on the inter- face.')
intSrvIfAttribAllocatedBuffer = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 1, 1, 3), BurstSize()).setUnits('Bytes').setMaxAccess("readonly")
if mibBuilder.loadTexts: intSrvIfAttribAllocatedBuffer.setStatus('current')
if mibBuilder.loadTexts: intSrvIfAttribAllocatedBuffer.setDescription('The amount of buffer space required to hold the simultaneous burst of all reserved flows on the interface.')
intSrvIfAttribFlows = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 1, 1, 4), Gauge32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: intSrvIfAttribFlows.setStatus('current')
if mibBuilder.loadTexts: intSrvIfAttribFlows.setDescription('The number of reserved flows currently active on this interface. A flow can be created ei- ther from a reservation protocol (such as RSVP or ST-II) or via configuration information.')
intSrvIfAttribPropagationDelay = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 1, 1, 5), Integer32()).setUnits('microseconds').setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvIfAttribPropagationDelay.setStatus('current')
if mibBuilder.loadTexts: intSrvIfAttribPropagationDelay.setDescription('The amount of propagation delay that this in- terface introduces in addition to that intro- diced by bit propagation delays.')
intSrvIfAttribStatus = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 1, 1, 6), RowStatus()).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvIfAttribStatus.setStatus('current')
if mibBuilder.loadTexts: intSrvIfAttribStatus.setDescription("'active' on interfaces that are configured for RSVP.")
intSrvFlowTable = MibTable((1, 3, 6, 1, 2, 1, 52, 1, 2), )
if mibBuilder.loadTexts: intSrvFlowTable.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowTable.setDescription("Information describing the reserved flows us- ing the system's interfaces.")
intSrvFlowEntry = MibTableRow((1, 3, 6, 1, 2, 1, 52, 1, 2, 1), ).setIndexNames((0, "INTEGRATED-SERVICES-MIB", "intSrvFlowNumber"))
if mibBuilder.loadTexts: intSrvFlowEntry.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowEntry.setDescription('Information describing the use of a given in- terface by a given flow. The counter intSrvFlowPoliced starts counting at the in- stallation of the flow.')
intSrvFlowNumber = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 1), SessionNumber())
if mibBuilder.loadTexts: intSrvFlowNumber.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowNumber.setDescription('The number of this flow. This is for SNMP In- dexing purposes only and has no relation to any protocol value.')
intSrvFlowType = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 2), SessionType()).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowType.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowType.setDescription('The type of session (IP4, IP6, IP6 with flow information, etc).')
intSrvFlowOwner = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 3), Integer32().subtype(subtypeSpec=ConstraintsUnion(SingleValueConstraint(1, 2, 3))).clone(namedValues=NamedValues(("other", 1), ("rsvp", 2), ("management", 3)))).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowOwner.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowOwner.setDescription('The process that installed this flow in the queue policy database.')
intSrvFlowDestAddr = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 4), OctetString().subtype(subtypeSpec=ValueSizeConstraint(4, 16))).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowDestAddr.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowDestAddr.setDescription("The destination address used by all senders in this session. This object may not be changed when the value of the RowStatus object is 'ac- tive'.")
intSrvFlowSenderAddr = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 5), OctetString().subtype(subtypeSpec=ValueSizeConstraint(4, 16))).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowSenderAddr.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowSenderAddr.setDescription("The source address of the sender selected by this reservation. The value of all zeroes in- dicates 'all senders'. This object may not be changed when the value of the RowStatus object is 'active'.")
intSrvFlowDestAddrLength = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 6), Integer32().subtype(subtypeSpec=ValueRangeConstraint(0, 128))).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowDestAddrLength.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowDestAddrLength.setDescription("The length of the destination address in bits. This is the CIDR Prefix Length, which for IP4 hosts and multicast addresses is 32 bits. This object may not be changed when the value of the RowStatus object is 'active'.")
intSrvFlowSenderAddrLength = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 7), Integer32().subtype(subtypeSpec=ValueRangeConstraint(0, 128))).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowSenderAddrLength.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowSenderAddrLength.setDescription("The length of the sender's address in bits. This is the CIDR Prefix Length, which for IP4 hosts and multicast addresses is 32 bits. This object may not be changed when the value of the RowStatus object is 'active'.")
intSrvFlowProtocol = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 8), Protocol()).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowProtocol.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowProtocol.setDescription("The IP Protocol used by a session. This ob- ject may not be changed when the value of the RowStatus object is 'active'.")
intSrvFlowDestPort = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 9), Port()).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowDestPort.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowDestPort.setDescription("The UDP or TCP port number used as a destina- tion port for all senders in this session. If the IP protocol in use, specified by intSrvResvFwdProtocol, is 50 (ESP) or 51 (AH), this represents a virtual destination port number. A value of zero indicates that the IP protocol in use does not have ports. This ob- ject may not be changed when the value of the RowStatus object is 'active'.")
intSrvFlowPort = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 10), Port()).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowPort.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowPort.setDescription("The UDP or TCP port number used as a source port for this sender in this session. If the IP protocol in use, specified by intSrvResvFwdProtocol is 50 (ESP) or 51 (AH), this represents a generalized port identifier (GPI). A value of zero indicates that the IP protocol in use does not have ports. This ob- ject may not be changed when the value of the RowStatus object is 'active'.")
intSrvFlowFlowId = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 11), Integer32().subtype(subtypeSpec=ValueRangeConstraint(0, 16777215))).setMaxAccess("readonly")
if mibBuilder.loadTexts: intSrvFlowFlowId.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowFlowId.setDescription('The flow ID that this sender is using, if this is an IPv6 session.')
intSrvFlowInterface = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 12), InterfaceIndex()).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowInterface.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowInterface.setDescription('The ifIndex value of the interface on which this reservation exists.')
intSrvFlowIfAddr = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 13), OctetString().subtype(subtypeSpec=ValueSizeConstraint(4, 16))).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowIfAddr.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowIfAddr.setDescription('The IP Address on the ifEntry on which this reservation exists. This is present primarily to support those interfaces which layer multi- ple IP Addresses on the interface.')
intSrvFlowRate = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 14), BitRate()).setUnits('bits per second').setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowRate.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowRate.setDescription("The Reserved Rate of the sender's data stream. If this is a Controlled Load service flow, this rate is derived from the Tspec rate parameter (r). If this is a Guaranteed service flow, this rate is derived from the Rspec clearing rate parameter (R).")
intSrvFlowBurst = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 15), BurstSize()).setUnits('bytes').setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowBurst.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowBurst.setDescription("The size of the largest burst expected from the sender at a time. If this is less than the sender's advertised burst size, the receiver is asking the network to provide flow pacing beyond what would be provided under normal circumstances. Such pac- ing is at the network's option.")
intSrvFlowWeight = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 16), Integer32()).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowWeight.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowWeight.setDescription('The weight used to prioritize the traffic. Note that the interpretation of this object is implementation-specific, as implementations vary in their use of weighting procedures.')
intSrvFlowQueue = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 17), Integer32()).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowQueue.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowQueue.setDescription('The number of the queue used by this traffic. Note that the interpretation of this object is implementation-specific, as implementations vary in their use of queue identifiers.')
intSrvFlowMinTU = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 18), MessageSize()).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowMinTU.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowMinTU.setDescription('The minimum message size for this flow. The policing algorithm will treat smaller messages as though they are this size.')
intSrvFlowMaxTU = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 19), MessageSize()).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowMaxTU.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowMaxTU.setDescription('The maximum datagram size for this flow that will conform to the traffic specification. This value cannot exceed the MTU of the interface.')
intSrvFlowBestEffort = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 20), Counter32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: intSrvFlowBestEffort.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowBestEffort.setDescription('The number of packets that were remanded to best effort service.')
intSrvFlowPoliced = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 21), Counter32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: intSrvFlowPoliced.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowPoliced.setDescription("The number of packets policed since the incep- tion of the flow's service.")
intSrvFlowDiscard = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 22), TruthValue().clone('false')).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowDiscard.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowDiscard.setDescription("If 'true', the flow is to incur loss when traffic is policed. If 'false', policed traff- ic is treated as best effort traffic.")
intSrvFlowService = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 23), QosService()).setMaxAccess("readonly")
if mibBuilder.loadTexts: intSrvFlowService.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowService.setDescription('The QoS service being applied to this flow.')
intSrvFlowOrder = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 24), Integer32().subtype(subtypeSpec=ValueRangeConstraint(0, 65535))).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowOrder.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowOrder.setDescription('In the event of ambiguity, the order in which the classifier should make its comparisons. The row with intSrvFlowOrder=0 is tried first, and comparisons proceed in the order of in- creasing value. Non-serial implementations of the classifier should emulate this behavior.')
intSrvFlowStatus = MibTableColumn((1, 3, 6, 1, 2, 1, 52, 1, 2, 1, 25), RowStatus()).setMaxAccess("readcreate")
if mibBuilder.loadTexts: intSrvFlowStatus.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowStatus.setDescription("'active' for all active flows. This object may be used to install static classifier infor- mation, delete classifier information, or au- thorize such.")
intSrvFlowNewIndex = MibScalar((1, 3, 6, 1, 2, 1, 52, 2, 1), TestAndIncr()).setMaxAccess("readwrite")
if mibBuilder.loadTexts: intSrvFlowNewIndex.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowNewIndex.setDescription("This object is used to assign values to intSrvFlowNumber as described in 'Textual Con- ventions for SNMPv2'. The network manager reads the object, and then writes the value back in the SET that creates a new instance of intSrvFlowEntry. If the SET fails with the code 'inconsistentValue', then the process must be repeated; If the SET succeeds, then the ob- ject is incremented, and the new instance is created according to the manager's directions.")
intSrvGroups = MibIdentifier((1, 3, 6, 1, 2, 1, 52, 4, 1))
intSrvCompliances = MibIdentifier((1, 3, 6, 1, 2, 1, 52, 4, 2))
intSrvCompliance = ModuleCompliance((1, 3, 6, 1, 2, 1, 52, 4, 2, 1)).setObjects(("INTEGRATED-SERVICES-MIB", "intSrvIfAttribGroup"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowsGroup"))

if getattr(mibBuilder, 'version', (0, 0, 0)) > (4, 4, 0):
    intSrvCompliance = intSrvCompliance.setStatus('current')
if mibBuilder.loadTexts: intSrvCompliance.setDescription('The compliance statement ')
intSrvIfAttribGroup = ObjectGroup((1, 3, 6, 1, 2, 1, 52, 4, 1, 1)).setObjects(("INTEGRATED-SERVICES-MIB", "intSrvIfAttribAllocatedBits"), ("INTEGRATED-SERVICES-MIB", "intSrvIfAttribMaxAllocatedBits"), ("INTEGRATED-SERVICES-MIB", "intSrvIfAttribAllocatedBuffer"), ("INTEGRATED-SERVICES-MIB", "intSrvIfAttribFlows"), ("INTEGRATED-SERVICES-MIB", "intSrvIfAttribPropagationDelay"), ("INTEGRATED-SERVICES-MIB", "intSrvIfAttribStatus"))
if getattr(mibBuilder, 'version', (0, 0, 0)) > (4, 4, 0):
    intSrvIfAttribGroup = intSrvIfAttribGroup.setStatus('current')
if mibBuilder.loadTexts: intSrvIfAttribGroup.setDescription('These objects are required for Systems sup- porting the Integrated Services Architecture.')
intSrvFlowsGroup = ObjectGroup((1, 3, 6, 1, 2, 1, 52, 4, 1, 2)).setObjects(("INTEGRATED-SERVICES-MIB", "intSrvFlowType"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowOwner"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowDestAddr"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowSenderAddr"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowDestAddrLength"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowSenderAddrLength"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowProtocol"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowDestPort"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowPort"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowInterface"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowBestEffort"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowRate"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowBurst"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowWeight"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowQueue"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowMinTU"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowDiscard"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowPoliced"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowService"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowIfAddr"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowOrder"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowStatus"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowFlowId"), ("INTEGRATED-SERVICES-MIB", "intSrvFlowMaxTU"))
if getattr(mibBuilder, 'version', (0, 0, 0)) > (4, 4, 0):
    intSrvFlowsGroup = intSrvFlowsGroup.setStatus('current')
if mibBuilder.loadTexts: intSrvFlowsGroup.setDescription('These objects are required for Systems sup- porting the Integrated Services Architecture.')
mibBuilder.exportSymbols("INTEGRATED-SERVICES-MIB", intSrvFlowMaxTU=intSrvFlowMaxTU, intSrvCompliances=intSrvCompliances, intSrvIfAttribStatus=intSrvIfAttribStatus, intSrvFlowPoliced=intSrvFlowPoliced, BurstSize=BurstSize, intSrvFlowMinTU=intSrvFlowMinTU, MessageSize=MessageSize, intSrvFlowEntry=intSrvFlowEntry, intSrvFlowDestPort=intSrvFlowDestPort, intSrvFlowProtocol=intSrvFlowProtocol, intSrvIfAttribMaxAllocatedBits=intSrvIfAttribMaxAllocatedBits, intSrvFlowDestAddrLength=intSrvFlowDestAddrLength, intSrvIfAttribGroup=intSrvIfAttribGroup, intSrvFlowOwner=intSrvFlowOwner, intSrvFlowStatus=intSrvFlowStatus, intSrvFlowWeight=intSrvFlowWeight, intSrvFlowBestEffort=intSrvFlowBestEffort, intSrvFlowInterface=intSrvFlowInterface, Protocol=Protocol, intSrvConformance=intSrvConformance, intSrvIfAttribFlows=intSrvIfAttribFlows, intSrvFlowsGroup=intSrvFlowsGroup, intSrvFlowSenderAddrLength=intSrvFlowSenderAddrLength, intSrvNotifications=intSrvNotifications, intSrvIfAttribTable=intSrvIfAttribTable, intSrvCompliance=intSrvCompliance, intSrv=intSrv, intSrvFlowNewIndex=intSrvFlowNewIndex, intSrvFlowPort=intSrvFlowPort, intSrvIfAttribPropagationDelay=intSrvIfAttribPropagationDelay, intSrvFlowService=intSrvFlowService, SessionNumber=SessionNumber, intSrvFlowType=intSrvFlowType, intSrvFlowRate=intSrvFlowRate, SessionType=SessionType, Port=Port, intSrvFlowDiscard=intSrvFlowDiscard, intSrvIfAttribAllocatedBits=intSrvIfAttribAllocatedBits, intSrvObjects=intSrvObjects, PYSNMP_MODULE_ID=intSrv, QosService=QosService, intSrvIfAttribAllocatedBuffer=intSrvIfAttribAllocatedBuffer, intSrvFlowSenderAddr=intSrvFlowSenderAddr, intSrvFlowTable=intSrvFlowTable, intSrvIfAttribEntry=intSrvIfAttribEntry, intSrvFlowDestAddr=intSrvFlowDestAddr, intSrvFlowQueue=intSrvFlowQueue, intSrvFlowOrder=intSrvFlowOrder, intSrvGenObjects=intSrvGenObjects, intSrvGroups=intSrvGroups, intSrvFlowNumber=intSrvFlowNumber, intSrvFlowFlowId=intSrvFlowFlowId, intSrvFlowBurst=intSrvFlowBurst, intSrvFlowIfAddr=intSrvFlowIfAddr, BitRate=BitRate)



# SNMP MIB module (ZINO-MIB) expressed in pysnmp data model.
#
# This Python module is designed to be imported and executed by the
# pysnmp library.
#
# See https://www.pysnmp.com/pysnmp for further information.
#
# Notes
# -----
# ASN.1 source file://./ZINO-MIB.my
# Produced by pysmi-1.6.2 at Thu Oct  2 13:15:57 2025
# On host agrajag platform Linux version 6.12.49 by user mvold
# Using Python version 3.12.7 (main, Oct 16 2024, 04:37:19) [Clang 18.1.8 ]

if 'mibBuilder' not in globals():
    import sys

    sys.stderr.write(__doc__)
    sys.exit(1)

# Import base ASN.1 objects even if this MIB does not use it

(Integer,
 OctetString,
 ObjectIdentifier) = mibBuilder.importSymbols(
    "ASN1",
    "Integer",
    "OctetString",
    "ObjectIdentifier")

(NamedValues,) = mibBuilder.importSymbols(
    "ASN1-ENUMERATION",
    "NamedValues")
(ConstraintsIntersection,
 ConstraintsUnion,
 SingleValueConstraint,
 ValueRangeConstraint,
 ValueSizeConstraint) = mibBuilder.importSymbols(
    "ASN1-REFINEMENT",
    "ConstraintsIntersection",
    "ConstraintsUnion",
    "SingleValueConstraint",
    "ValueRangeConstraint",
    "ValueSizeConstraint")

# Import SMI symbols from the MIBs this MIB depends on

(ModuleCompliance,
 NotificationGroup) = mibBuilder.importSymbols(
    "SNMPv2-CONF",
    "ModuleCompliance",
    "NotificationGroup")

(Bits,
 Counter32,
 Counter64,
 Gauge32,
 Integer32,
 IpAddress,
 ModuleIdentity,
 MibIdentifier,
 NotificationType,
 ObjectIdentity,
 MibScalar,
 MibTable,
 MibTableRow,
 MibTableColumn,
 TimeTicks,
 Unsigned32,
 iso) = mibBuilder.importSymbols(
    "SNMPv2-SMI",
    "Bits",
    "Counter32",
    "Counter64",
    "Gauge32",
    "Integer32",
    "IpAddress",
    "ModuleIdentity",
    "MibIdentifier",
    "NotificationType",
    "ObjectIdentity",
    "MibScalar",
    "MibTable",
    "MibTableRow",
    "MibTableColumn",
    "TimeTicks",
    "Unsigned32",
    "iso")

(DateAndTime,
 DisplayString,
 PhysAddress,
 TextualConvention,
 TimeInterval,
 TimeStamp) = mibBuilder.importSymbols(
    "SNMPv2-TC",
    "DateAndTime",
    "DisplayString",
    "PhysAddress",
    "TextualConvention",
    "TimeInterval",
    "TimeStamp")

(uninettMibs,
 zinoMIB) = mibBuilder.importSymbols(
    "UNINETT-SMI",
    "uninettMibs",
    "zinoMIB")


# MODULE-IDENTITY

zinoMIB = ModuleIdentity(
    (1, 3, 6, 1, 4, 1, 2428, 130, 1)
)
if mibBuilder.loadTexts:
    zinoMIB.setLastUpdated("201201190000Z")
if mibBuilder.loadTexts:
    zinoMIB.setOrganization("UNINETT AS")
if mibBuilder.loadTexts:
    zinoMIB.setContactInfo(" UNINETT AS NO-7465 Trondheim Norway E-mail: drift@uninett.no")
if mibBuilder.loadTexts:
    zinoMIB.setDescription("The MIB for zino keepalive / polling.")


# Types definitions


# TEXTUAL-CONVENTIONS



# MIB Managed Objects in the order of their OIDs

_ZinoMIBObjects_ObjectIdentity = ObjectIdentity
zinoMIBObjects = _ZinoMIBObjects_ObjectIdentity(
    (1, 3, 6, 1, 4, 1, 2428, 130, 1, 1)
)
_ZinoUpTime_Type = Counter32
_ZinoUpTime_Object = MibScalar
zinoUpTime = _ZinoUpTime_Object(
    (1, 3, 6, 1, 4, 1, 2428, 130, 1, 1, 1),
    _ZinoUpTime_Type()
)
zinoUpTime.setMaxAccess("read-only")
if mibBuilder.loadTexts:
    zinoUpTime.setStatus("mandatory")
if mibBuilder.loadTexts:
    zinoUpTime.setDescription("The time (in seconds) since this zino server instance was re-initialized.")

# Managed Objects groups


# Notification objects


# Notifications groups


# Agent capabilities


# Module compliance


# Export all MIB objects to the MIB builder

mibBuilder.exportSymbols(
    "ZINO-MIB",
    **{"zinoMIB": zinoMIB,
       "zinoMIBObjects": zinoMIBObjects,
       "zinoUpTime": zinoUpTime}
)



"""OID manipulation"""

SEPARATOR = "."
SEPARATOR_B = b"."


class OID(tuple):
    """Object IDentifier represented in tuple form.

    Example usages:

      >>> ifXTable = OID('.1.3.6.1.2.1.31.1.1')
      >>> ifXTable
      OID('.1.3.6.1.2.1.31.1.1')
      >>> ifName = ifXTable + '1.1'
      >>> ifName
      OID('.1.3.6.1.2.1.31.1.1.1.1')
      >>> ifXTable.is_a_prefix_of(ifName)
      True
      >>> ifName.strip_prefix(ifXTable)
      OID('.1.1')
      >>> str(ifXTable)
      '.1.3.6.1.2.1.31.1.1'
      >>> ifXTable[:3]
      (1, 3, 6)

    """

    def __new__(cls, oid):
        if isinstance(oid, str):
            oid = map(int, oid.strip(SEPARATOR).split(SEPARATOR))
        elif isinstance(oid, bytes):
            oid = map(int, oid.strip(SEPARATOR_B).split(SEPARATOR_B))
        elif isinstance(oid, OID):
            return oid
        return tuple.__new__(cls, oid)

    def __init__(self, oid):
        super().__init__()
        if not all(isinstance(i, int) for i in self):
            raise TypeError("Not all arguments could be converted to `int`")

    def __str__(self):
        return SEPARATOR + SEPARATOR.join([str(i) for i in self])

    def __repr__(self):
        return "OID(%s)" % repr(str(self))

    def __add__(self, other):
        return OID(super(OID, self).__add__(OID(other)))

    def is_a_prefix_of(self, other):
        """Returns True if this OID is a prefix of other"""
        other = OID(other)
        return len(other) > len(self) and other[: len(self)] == self

    def strip_prefix(self, prefix):
        """Returns this OID with prefix stripped.

        If prefix isn't an actual prefix of this OID, this OID is returned
        unchanged.

        """
        prefix = OID(prefix)
        if prefix.is_a_prefix_of(self):
            return OID(self[len(prefix) :])
        else:
            return self



import logging
from typing import Optional, Tuple

from zino.tasks.task import Task

_logger = logging.getLogger(__name__)
ENTERPRISES = (1, 3, 6, 1, 4, 1)


class VendorTask(Task):
    """Fetches and stores state information about a Device's vendor"""

    async def run(self):
        vendor = await self._get_enterprise_id()
        _logger.debug("%s enterprise id: %r", self.device.name, vendor)

        if self.device_state.enterprise_id != vendor:
            _logger.info(
                "%s changed enterprise id from %s to %s", self.device.name, self.device_state.enterprise_id, vendor
            )
            self.device_state.enterprise_id = vendor

    async def _get_enterprise_id(self) -> Optional[int]:
        sysobjectid = await self._get_sysobjectid()
        # This part can probably be a whole lot prettier if we learned how to utilize PySNMP properly:
        if sysobjectid[: len(ENTERPRISES)] == ENTERPRISES:
            return sysobjectid[len(ENTERPRISES)]

    async def _get_sysobjectid(self) -> Optional[Tuple[int, ...]]:
        result = await self.snmp.get("SNMPv2-MIB", "sysObjectID", 0)
        return result.value



from datetime import timedelta
from unittest.mock import Mock

import pytest

from zino.state import ZinoState
from zino.statemodels import (
    AlarmEvent,
    DeviceMaintenance,
    EventState,
    MatchType,
    Port,
    PortStateEvent,
    PortStateMaintenance,
    ReachabilityEvent,
)
from zino.time import now


def test_should_start_with_no_planned_maintenances(pms):
    assert len(pms) == 0


def test_pm_should_be_gettable_by_id(pms, active_device_pm):
    assert pms[active_device_pm.id] == active_device_pm


class TestGetStartedPlannedMaintenances:
    def test_should_return_pms_that_started_since_last_run(self, pms, recent_pm):
        pms.last_run = now() - timedelta(hours=1)
        started_pms = pms.get_started_planned_maintenances(now=now())
        assert recent_pm in started_pms

    def test_should_not_return_pms_that_started_before_last_run(self, pms, active_device_pm):
        pms.last_run = now() - timedelta(hours=1)
        started_pms = pms.get_started_planned_maintenances(now=now())
        assert active_device_pm not in started_pms

    def test_should_not_return_pms_that_have_not_started_yet(self, pms, not_started_pm):
        pms.last_run = now() - timedelta(hours=1)
        started_pms = pms.get_started_planned_maintenances(now=now())
        assert not_started_pm not in started_pms


class TestGetEndedPlannedMaintenances:
    def test_should_return_pms_that_ended_after_last_run(self, pms, active_device_pm, ended_pm, old_pm):
        pms.last_run = now() - timedelta(hours=1)
        ended_pms = pms.get_ended_planned_maintenances(now=now())
        assert ended_pm in ended_pms

    def test_should_not_return_pms_that_ended_before_last_run(self, pms, old_pm):
        pms.last_run = now() - timedelta(hours=1)
        ended_pms = pms.get_ended_planned_maintenances(now=now())
        assert old_pm not in ended_pms

    def test_should_not_return_pms_that_have_not_ended(self, pms, active_device_pm):
        pms.last_run = now() - timedelta(hours=1)
        ended_pms = pms.get_ended_planned_maintenances(now=now())
        assert active_device_pm not in ended_pms


class TestGetActivePlannedMaintenances:
    def test_should_return_active_pms(self, pms, active_device_pm):
        active_pms = pms.get_active_planned_maintenances(now())
        assert active_device_pm in active_pms

    def test_should_not_return_ended_pms(self, pms, ended_pm):
        active_pms = pms.get_active_planned_maintenances(now())
        assert ended_pm not in active_pms

    def test_should_not_return_pms_that_have_not_started_yet(self, pms, not_started_pm):
        active_pms = pms.get_active_planned_maintenances(now())
        assert not_started_pm not in active_pms


class TestGetOldPlannedMaintenances:
    def test_should_return_old_pms(self, pms, old_pm):
        old_pms = pms.get_old_planned_maintenances(now=now())
        assert old_pm in old_pms

    def test_should_not_return_pms_that_have_not_ended_yet(self, pms, active_device_pm):
        old_pms = pms.get_old_planned_maintenances(now=now())
        assert active_device_pm not in old_pms

    def test_should_not_return_pms_that_ended_since_last_run(self, pms, ended_pm):
        old_pms = pms.get_old_planned_maintenances(now=now())
        assert ended_pm not in old_pms


class TestClosePlannedMaintenance:
    def test_existing_pm_should_be_deleted(self, pms, old_pm):
        pms.close_planned_maintenance(old_pm.id, "test", "test")
        assert old_pm.id not in pms.planned_maintenances

    def test_when_there_are_no_matching_pms_it_should_not_raise_exception(self, pms, old_pm):
        assert pms.close_planned_maintenance(pms.get_next_available_pm_id(), "test", "test") is None

    def test_should_call_observers_after_closing_pm(self, pms, old_pm):
        observer = Mock()
        pms.add_pm_observer(observer.observe)
        pms.close_planned_maintenance(old_pm.id, "test", "test")
        assert observer.observe.called


class TestUpdatePmStates:
    def test_events_matching_active_device_pm_should_be_set_to_ignored(self, state, active_device_pm):
        device = state.devices.get("device")
        state.planned_maintenances.update_pm_states(state)
        reachability_event = state.events.get(device.name, None, ReachabilityEvent)
        assert reachability_event.state == EventState.IGNORED
        yellow_alarm_event = state.events.get(device.name, "yellow", AlarmEvent)
        assert yellow_alarm_event.state == EventState.IGNORED
        red_alarm_event = state.events.get(device.name, "red", AlarmEvent)
        assert red_alarm_event.state == EventState.IGNORED

    def test_events_matching_active_portstate_pm_should_be_set_to_ignored(self, state, active_portstate_pm):
        device = state.devices.get("device")
        port = Port(ifindex=1, ifdescr="port", ifalias="portalias")
        device.ports[port.ifindex] = port
        state.planned_maintenances.update_pm_states(state)
        event = state.events.get(device.name, port.ifindex, PortStateEvent)
        assert event.state == EventState.IGNORED

    def test_events_matching_active_portstate_pm_with_regexp_matching_not_at_the_beginning_should_be_set_to_ignored(
        self, state, active_portstate_pm
    ):
        device = state.devices.get("blabla_device")
        port = Port(ifindex=1, ifdescr="port", ifalias="portalias")
        device.ports[port.ifindex] = port
        state.planned_maintenances.update_pm_states(state)
        event = state.events.get(device.name, port.ifindex, PortStateEvent)
        assert event.state == EventState.IGNORED

    def test_when_pm_ends_its_affected_events_should_be_opened(self, state, ended_pm):
        device = state.devices.get("device")

        # Create event and register it as affected by PM
        reachability_event = state.events.create_event(device.name, None, ReachabilityEvent)
        reachability_event.state = EventState.IGNORED
        state.events.commit(reachability_event)
        ended_pm.event_ids.append(reachability_event.id)

        state.planned_maintenances.update_pm_states(state)
        assert state.events.checkout(reachability_event.id).state == EventState.OPEN

    def test_old_pms_should_be_deleted(self, state, old_pm):
        assert old_pm.id in state.planned_maintenances.planned_maintenances
        state.planned_maintenances.update_pm_states(state)
        assert old_pm.id not in state.planned_maintenances.planned_maintenances

    def test_event_opened_after_pm_was_initiated_should_be_set_to_ignored(self, state, active_device_pm):
        device = state.devices.get("device")
        event = state.events.create_event(device.name, None, ReachabilityEvent)
        event.state = EventState.OPEN
        state.events.commit(event)
        # Set last run to be after start time so PM is not treated as if it just started
        state.planned_maintenances.last_run = active_device_pm.start_time + timedelta(hours=1)
        state.planned_maintenances.update_pm_states(state)

    def test_events_matching_active_device_pm_with_regexp_matching_at_the_beginning_should_be_set_to_ignored(
        self, state, active_regexp_device_pm
    ):
        device = state.devices.get("device")
        state.planned_maintenances.update_pm_states(state)
        reachability_event = state.events.get(device.name, None, ReachabilityEvent)
        assert reachability_event.state == EventState.IGNORED
        yellow_alarm_event = state.events.get(device.name, "yellow", AlarmEvent)
        assert yellow_alarm_event.state == EventState.IGNORED
        red_alarm_event = state.events.get(device.name, "red", AlarmEvent)
        assert red_alarm_event.state == EventState.IGNORED

    def test_events_matching_active_device_with_regexp_matching_not_at_the_beginning_should_be_set_to_ignored(
        self, state, active_regexp_device_pm
    ):
        device = state.devices.get("blabla_device")
        state.planned_maintenances.update_pm_states(state)
        reachability_event = state.events.get(device.name, None, ReachabilityEvent)
        assert reachability_event.state == EventState.IGNORED
        yellow_alarm_event = state.events.get(device.name, "yellow", AlarmEvent)
        assert yellow_alarm_event.state == EventState.IGNORED
        red_alarm_event = state.events.get(device.name, "red", AlarmEvent)
        assert red_alarm_event.state == EventState.IGNORED


def test_pms_should_be_parsed_as_correct_subclass_when_read_from_file(
    tmp_path, state, active_portstate_pm, active_device_pm
):
    dumpfile = tmp_path / "dump.json"
    state.dump_state_to_file(dumpfile)
    read_state = ZinoState.load_state_from_file(str(dumpfile))
    read_device_pm = read_state.planned_maintenances[active_device_pm.id]
    read_portstate_pm = read_state.planned_maintenances[active_portstate_pm.id]
    assert isinstance(read_device_pm, DeviceMaintenance)
    assert isinstance(read_portstate_pm, PortStateMaintenance)


def test_model_dump_of_pm_should_use_aliases(state, active_portstate_pm, active_device_pm):
    state_dump = state.model_dump_json(exclude_none=True, indent=2, by_alias=True)
    assert "starttime" in state_dump
    assert "endtime" in state_dump
    assert "match_dev" in state_dump
    assert "match_expr" in state_dump


@pytest.fixture
def state(pms):
    state = ZinoState()
    state.planned_maintenances = pms
    return state


@pytest.fixture
def not_started_pm(pms):
    return pms.create_planned_maintenance(
        start_time=now() + timedelta(days=1),
        end_time=now() + timedelta(days=2),
        pm_class=DeviceMaintenance,
        match_type=MatchType.EXACT,
        match_expression="device",
        match_device="device",
    )


@pytest.fixture
def recent_pm(pms):
    return pms.create_planned_maintenance(
        start_time=now() - timedelta(minutes=1),
        end_time=now() + timedelta(days=1),
        pm_class=DeviceMaintenance,
        match_type=MatchType.STR,
        match_expression="hello",
        match_device="device",
    )


@pytest.fixture
def old_pm(pms):
    return pms.create_planned_maintenance(
        start_time=now() - timedelta(days=100),
        end_time=now() - timedelta(days=99),
        pm_class=DeviceMaintenance,
        match_type=MatchType.EXACT,
        match_expression="device",
        match_device="device",
    )


@pytest.fixture
def active_regexp_device_pm(pms):
    return pms.create_planned_maintenance(
        start_time=now() - timedelta(days=1),
        end_time=now() + timedelta(days=1),
        pm_class=DeviceMaintenance,
        match_type=MatchType.REGEXP,
        match_expression="device",
        match_device="device",
    )



from typing import Optional

try:
    from tomllib import TOMLDecodeError, load
except ImportError:
    from tomli import TOMLDecodeError, load

from .models import Configuration


class InvalidConfigurationError(Exception):
    """The configuration file is invalid toml"""


def read_configuration(config_file_name: str, poll_file_name: Optional[str] = None) -> Configuration:
    """
    Reads and validates config toml file

    Returns configuration if file name is given and file exists

    Raises InvalidConfigurationError if toml file is invalid,
    OSError if the config toml file could not be found and
    pydantic.ValidationError if values in it are invalid or the specified files
    don't exist
    """
    with open(config_file_name, mode="rb") as cf:
        try:
            config_dict = load(cf)
        except TOMLDecodeError:
            raise InvalidConfigurationError

    # Polldevs by command line argument will override config file entry
    if poll_file_name:
        if "polling" not in config_dict.keys():
            config_dict["polling"] = {"file": poll_file_name}
        else:
            config_dict["polling"]["file"] = poll_file_name

    config = Configuration.model_validate(obj=config_dict, strict=True)

    return config



from datetime import timedelta
from unittest.mock import patch

import pytest

from zino.config.models import Configuration, PollDevice
from zino.oid import OID
from zino.state import ZinoState
from zino.statemodels import InterfaceState, Port, PortStateEvent
from zino.tasks.linkstatetask import (
    BaseInterfaceRow,
    CollectedInterfaceDataIsNotSaneError,
    LinkStateTask,
    MissingInterfaceTableData,
)
from zino.time import now


class TestLinkStateTask:
    async def test_run_should_not_create_event_if_links_are_up(self, linkstatetask_with_links_up):
        task = linkstatetask_with_links_up
        assert (await task.run()) is None
        assert len(task.state.events) == 0

    async def test_given_event_suppression_config_of_true_when_one_link_is_down_then_run_should_create_event(
        self, linkstatetask_with_one_link_down
    ):
        task = linkstatetask_with_one_link_down
        task.config.event.make_events_for_new_interfaces = True
        assert (await task.run()) is None
        assert len(task.state.events) == 1

    async def test_given_event_suppression_config_of_false_when_one_link_is_down_then_run_should_not_create_event(
        self, linkstatetask_with_one_link_down
    ):
        task = linkstatetask_with_one_link_down
        task.config.event.make_events_for_new_interfaces = False
        assert (await task.run()) is None
        assert len(task.state.events) == 0

    def test_when_patterns_are_empty_interface_should_not_be_ignored(self, task_with_dummy_device):
        data = BaseInterfaceRow(
            index=2, descr="GigabitEthernet1/2", alias="uplink", admin_status="up", oper_status="up", last_change=0
        )
        assert task_with_dummy_device._is_interface_watched(data)

    def test_when_interface_matches_watchpat_it_should_not_be_ignored(self, task_with_dummy_device):
        data = BaseInterfaceRow(
            index=2, descr="GigabitEthernet1/2", alias="uplink", admin_status="up", oper_status="up", last_change=0
        )
        task_with_dummy_device.device.watchpat = "Gigabit"
        assert task_with_dummy_device._is_interface_watched(data)

    def test_when_interface_matches_watchpat_not_at_the_beginning_it_should_not_be_ignored(
        self, task_with_dummy_device
    ):
        data = BaseInterfaceRow(
            index=2, descr="GigabitEthernet1/2", alias="uplink", admin_status="up", oper_status="up", last_change=0
        )
        task_with_dummy_device.device.watchpat = "1/2"
        assert task_with_dummy_device._is_interface_watched(data)

    def test_when_interface_doesnt_match_watchpat_it_should_be_ignored(self, task_with_dummy_device):
        data = BaseInterfaceRow(
            index=2, descr="GigabitEthernet1/2", alias="uplink", admin_status="up", oper_status="up", last_change=0
        )
        task_with_dummy_device.device.watchpat = "TenGiga"
        assert not task_with_dummy_device._is_interface_watched(data)

    def test_when_interface_matches_ignorepat_it_should_be_ignored(self, task_with_dummy_device):
        data = BaseInterfaceRow(
            index=2, descr="GigabitEthernet1/2", alias="uplink", admin_status="up", oper_status="up", last_change=0
        )
        task_with_dummy_device.device.ignorepat = ".*Ethernet"
        assert not task_with_dummy_device._is_interface_watched(data)

    def test_when_interface_matches_ignorepat_not_at_the_beginning_it_should_be_ignored(self, task_with_dummy_device):
        data = BaseInterfaceRow(
            index=2, descr="GigabitEthernet1/2", alias="uplink", admin_status="up", oper_status="up", last_change=0
        )
        task_with_dummy_device.device.ignorepat = "1/2"
        assert not task_with_dummy_device._is_interface_watched(data)

    def test_when_interface_state_is_missing_update_state_should_raise_exception(self, task_with_dummy_device):
        row = BaseInterfaceRow(index=42, descr="x", alias="x", admin_status="x", oper_status="x", last_change=0)
        port = Port(ifindex=42)
        empty_state_row = {}

        with pytest.raises(MissingInterfaceTableData):
            task_with_dummy_device._update_state(data=row, port=port, row=empty_state_row)

    def test_when_interface_data_is_empty_update_single_interface_should_raise_exception(self, task_with_dummy_device):
        with pytest.raises(CollectedInterfaceDataIsNotSaneError):
            task_with_dummy_device._update_single_interface({})

    def test_when_interface_data_is_empty_update_interfaces_should_keep_processing(self, task_with_dummy_device):
        assert (
            task_with_dummy_device._update_interfaces(
                {
                    OID(".1"): {},
                    OID(".2"): {},
                    OID(".3"): {},
                }
            )
            is None
        )

    @pytest.mark.asyncio
    async def test_when_event_is_new_it_should_set_lasttrans(self, linkstatetask_with_one_link_down):
        task = linkstatetask_with_one_link_down
        task.config.event.make_events_for_new_interfaces = True
        await task.run()
        event = task.state.events.get(task.device.name, 2, PortStateEvent)
        assert event.lasttrans

    @pytest.mark.asyncio
    async def test_when_event_transitions_from_to_up_it_should_update_lasttrans(self, linkstatetask_with_links_up):
        task = linkstatetask_with_links_up
        initial_lasttrans = now() - timedelta(minutes=5)

        device = task.device_state
        device.ports.update({1: Port(ifindex=1, ifdescr="1", ifalias="from a famous", state=InterfaceState.DOWN)})

        event = task.state.events.create_event(task.device.name, 1, PortStateEvent)
        event.ifindex = 1
        event.portstate = InterfaceState.DOWN
        event.lasttrans = initial_lasttrans
        task.state.events.commit(event)

        assert (await task.run()) is None
        updated_event = task.state.events[event.id]
        assert updated_event.portstate == InterfaceState.UP
        assert updated_event.lasttrans > initial_lasttrans

    @pytest.mark.asyncio
    async def test_when_event_transitions_from_down_to_admindown_it_should_update_lasttrans(
        self, linkstatetask_with_admin_down
    ):
        task = linkstatetask_with_admin_down
        initial_lasttrans = now() - timedelta(minutes=5)

        device = task.device_state
        device.ports.update({1: Port(ifindex=1, ifdescr="1", ifalias="from a famous", state=InterfaceState.DOWN)})

        event = task.state.events.create_event(task.device.name, 1, PortStateEvent)
        event.ifindex = 1
        event.portstate = InterfaceState.DOWN
        event.lasttrans = initial_lasttrans
        task.state.events.commit(event)

        assert (await task.run()) is None
        updated_event = task.state.events[event.id]
        assert updated_event.portstate == InterfaceState.ADMIN_DOWN
        assert updated_event.lasttrans > initial_lasttrans

    @pytest.mark.asyncio
    async def test_when_event_transitions_from_down_to_up_it_should_update_ac_down(self, linkstatetask_with_links_up):
        task = linkstatetask_with_links_up
        initial_lasttrans = now() - timedelta(minutes=5)
        initial_ac_down = timedelta(0)

        device = task.device_state
        device.ports.update({1: Port(ifindex=1, ifdescr="1", ifalias="from a famous", state=InterfaceState.DOWN)})

        event = task.state.events.create_event(task.device.name, 1, PortStateEvent)
        event.ifindex = 1
        event.portstate = InterfaceState.DOWN
        event.ac_down = initial_ac_down
        event.lasttrans = initial_lasttrans
        task.state.events.commit(event)

        assert (await task.run()) is None
        updated_event = task.state.events[event.id]
        assert updated_event.portstate == InterfaceState.UP
        assert updated_event.ac_down > initial_ac_down

    @pytest.mark.asyncio
    async def test_when_event_transitions_from_down_to_admindown_it_should_update_ac_down(
        self, linkstatetask_with_admin_down
    ):
        task = linkstatetask_with_admin_down
        initial_lasttrans = now() - timedelta(minutes=5)
        initial_ac_down = timedelta(0)

        device = task.device_state
        device.ports.update({1: Port(ifindex=1, ifdescr="1", ifalias="alias", state=InterfaceState.DOWN)})

        event = task.state.events.create_event(task.device.name, 1, PortStateEvent)
        event.ifindex = 1
        event.portstate = InterfaceState.DOWN
        event.ac_down = initial_ac_down
        event.lasttrans = initial_lasttrans
        task.state.events.commit(event)

        assert (await task.run()) is None
        updated_event = task.state.events[event.id]
        assert updated_event.portstate == InterfaceState.ADMIN_DOWN
        assert updated_event.ac_down > initial_ac_down


class TestBaseInterfaceRow:
    def test_when_index_is_missing_is_sane_should_return_false(self):
        row = BaseInterfaceRow(index=None, descr="x", alias="x", admin_status="x", oper_status="x", last_change=0)
        assert not row.is_sane()

    def test_when_descr_is_missing_is_sane_should_return_false(self):
        row = BaseInterfaceRow(index=42, descr=None, alias="x", admin_status="x", oper_status="x", last_change=0)
        assert not row.is_sane()

    def test_when_descr_and_index_are_present_is_sane_should_return_true(self):
        row = BaseInterfaceRow(index=42, descr="x", alias="x", admin_status="x", oper_status="x", last_change=0)
        assert row.is_sane()

    async def test_poll_single_interface_should_update_state(self, linkstatetask_with_one_link_down):
        target_index = 2
        await linkstatetask_with_one_link_down.poll_single_interface(target_index)
        device_state = linkstatetask_with_one_link_down.state.devices.get(linkstatetask_with_one_link_down.device.name)

        assert target_index in device_state.ports, f"no state for port {target_index} was stored"
        port = device_state.ports[target_index]
        assert port.state == InterfaceState.DOWN
        assert port.ifdescr == "2"
        assert port.ifalias == "from a famous"

    async def test_when_ifindex_is_1_poll_single_interface_should_not_crash(self, linkstatetask_with_one_link_down):
        """Regression test.  poll_single_interface constructs a GET-NEXT request by asking for the set of next values
        after ifindex-1 - but 0 indexes are illegal when encoding OIDs into tables.
        """
        target_index = 1
        assert await linkstatetask_with_one_link_down.poll_single_interface(target_index) is None

    async def test_when_timeout_occurs_poll_single_interface_should_not_crash(self, linkstatetask_with_one_link_down):
        with patch.object(linkstatetask_with_one_link_down.snmp, "getnext2") as mock_getnext2:
            mock_getnext2.side_effect = TimeoutError
            assert await linkstatetask_with_one_link_down.poll_single_interface(42) is None


@pytest.fixture
def linkstatetask_with_links_up(snmpsim, snmp_test_port):
    device = PollDevice(name="buick.lab.example.org", address="127.0.0.1", port=snmp_test_port, community="linksup")
    state = ZinoState()
    task = LinkStateTask(device, state, Configuration())
    yield task


@pytest.fixture
def linkstatetask_with_one_link_down(snmpsim, snmp_test_port):
    device = PollDevice(name="buick.lab.example.org", address="127.0.0.1", port=snmp_test_port, community="linksdown")
    state = ZinoState()
    task = LinkStateTask(device, state, Configuration())
    yield task


@pytest.fixture
def linkstatetask_with_admin_down(snmpsim, snmp_test_port):
    device = PollDevice(
        name="buick.lab.example.org", address="127.0.0.1", port=snmp_test_port, community="linksadmindown"
    )
    state = ZinoState()
    task = LinkStateTask(device, state, Configuration())
    yield task


@pytest.fixture
def task_with_dummy_device():
    device = PollDevice(name="test", address="127.0.0.1")
    state = ZinoState()
    task = LinkStateTask(device, state, Configuration())
    yield task



#!/usr/bin/env python3
"""Fetch SNMP agent uptime using Zino high-level APIs"""

import argparse
import asyncio
import logging

from zino.config.polldevs import read_polldevs
from zino.snmp import SNMP

_log = logging.getLogger(__name__)


def main():
    args = parse_args()
    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s - %(levelname)s - %(name)s (%(threadName)s) - %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S %Z",
    )
    asyncio.get_event_loop().run_until_complete(run(args))


async def run(args: argparse.Namespace):
    devices = {d.name: d for d in read_polldevs("polldevs.cf")}
    device = devices[args.router]

    snmp = SNMP(device)
    yellow_alarm_count = await snmp.get("JUNIPER-ALARM-MIB", "jnxYellowAlarmCount", 0)
    _log.info("Response from %s: %r (%s)", device.name, yellow_alarm_count, type(yellow_alarm_count))


def parse_args():
    devicenames = [d.name for d in read_polldevs("polldevs.cf")]
    parser = argparse.ArgumentParser(description="Fetch yellow alert count from a device in polldevs.cf")
    parser.add_argument("router", type=str, help="Zino router name", choices=devicenames)
    return parser.parse_args()


if __name__ == "__main__":
    main()
